/*
 * Originally written r. simic
 * (c) 2011 Industrie It Pty Ltd
 * http://www.industrieit.com/
 *
 * IndustrieIT Pty Ltd licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * The software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.industrieit.jsf.stateless.impl;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.faces.component.UIViewRoot;

public class SJSFStatePool {

	private static Map<String, LinkedList<UIViewRoot>> ppool=new Hashtable<String, LinkedList<UIViewRoot>>();
	
	private static ExecutorService exServ;
	
	static {
		ThreadFactory tf=new ThreadFactory() {
			
			public Thread newThread(Runnable r) {
				Thread t=new Thread(r);
				t.setDaemon(true);
				return t;
			}
		};
		
		BlockingQueue<Runnable> q=new  LinkedBlockingQueue<Runnable>(100);
		exServ=new ThreadPoolExecutor(1, 3, 30l, TimeUnit.SECONDS, q,tf,new ThreadPoolExecutor.CallerRunsPolicy());
	}
	
	public static synchronized UIViewRoot get(String uri)
	{
		LinkedList<UIViewRoot> items=getPoolForKey(uri);

		UIViewRoot v=null;
		synchronized(items)
		{
			if (items.isEmpty())
			{
				return null;
			}
			v= (UIViewRoot) items.removeFirst();
		}
		long intime=(Long) v.getAttributes().get(SJSFStatics.INPOOL);
		//System.out.println("delay "+(System.currentTimeMillis()-intime));
		return v;
	}
	
	public static synchronized void clearPool()
	{
		ppool=new HashMap<String, LinkedList<UIViewRoot>>();
	}
	
	private static LinkedList<UIViewRoot> getPoolForKey(String k)
	{
		LinkedList<UIViewRoot> items=(LinkedList<UIViewRoot>) ppool.get(k);
		if (items==null)
		{
			items=new LinkedList<UIViewRoot>();
			ppool.put(k, items);
		}
		return items;
	}
	
	public static synchronized void cache(String uri, UIViewRoot v)
	{
		LinkedList<UIViewRoot> items=getPoolForKey(uri);
		SJSFCleanTask t=new SJSFCleanTask(v,items);
		exServ.submit(t);
	}
	
	public static int getPoolCount(String uri)
	{
		LinkedList<UIViewRoot> items=getPoolForKey(uri);
		return items.size();
	}

}
